require "spec_helper"

describe InfoparkComponentCache::Guards::LastChanged do
  subject(:guard) { described_class.new(cache_component_stub) }

  let(:cache_component_stub) do
    double.tap do |cache_component_stub|
      cache_component_stub.stub(:cache_key) do |input|
        input.inspect
      end
    end
  end

  let(:maximum_last_changed) { 10.minutes.ago }

  let(:obj_stub) do
    double.tap do |obj_stub|
      obj_stub.stub(:scoped).and_return(obj_stub)
      obj_stub.stub(:where).and_return(obj_stub)
      obj_stub.stub(:maximum) do |field|
        maximum_last_changed if field == :last_changed
      end
    end
  end

  before { InfoparkComponentCache::CmsStateGuard.obj_root_class = obj_stub }

  context "with cache disabled" do
    disable_cache

    context "without a call to guard!" do
      it { is_expected.not_to be_consistent }
    end

    context "with a call to guard!" do
      before { guard.guard! }

      it { is_expected.not_to be_consistent }
    end
  end

  context "with cache enabled" do
    enable_cache
    before { Rails.cache.clear }

    context "without a call to guard!" do
      it { is_expected.not_to be_consistent }
    end

    context "with a call to guard!" do
      before { guard.guard! }

      it { is_expected.to be_consistent }
    end

    context "when after maximum last change changes" do
      before do
        guard.guard!
        obj_stub.stub(:maximum).and_return(Time.now.to_iso)
      end

      it { is_expected.not_to be_consistent }
    end

    context "when after maximum last change changes to a past date" do
      before do
        guard.guard!
        obj_stub.stub(:maximum).and_return(1.month.ago.to_iso)
      end

      it { is_expected.to be_consistent }
    end
  end
end
